(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./RedBlackTreeNode"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const RedBlackTreeNode_1 = require("./RedBlackTreeNode");
    class RedBlackTree {
        constructor(comparator = (a, b) => a > b) {
            this.root = RedBlackTreeNode_1.default.sentinel;
            this.sentinel = RedBlackTreeNode_1.default.sentinel;
            this.comparator = comparator;
        }
        add(value) {
            let parent = this.sentinel;
            let currentLoopNode = this.root;
            const newNode = new RedBlackTreeNode_1.default(value, RedBlackTreeNode_1.default.sentinel, false);
            while (currentLoopNode.isNotSentinel()) {
                parent = currentLoopNode;
                if (this.comparator(currentLoopNode.value, newNode.value)) {
                    currentLoopNode = currentLoopNode.left;
                }
                else {
                    currentLoopNode = currentLoopNode.right;
                }
            }
            newNode.parent = parent;
            if (parent.isSentinel()) {
                this.root = newNode;
            }
            else if (this.comparator(parent.value, newNode.value)) {
                parent.left = newNode;
            }
            else {
                parent.right = newNode;
            }
            this.addFixup(newNode);
        }
        remove(value) {
            return false;
        }
        inOrderTreeWalk(callback, initialValue) {
            return this.inorderNodeWalk(this.root, callback, initialValue);
        }
        isEmpty() {
            return this.root === RedBlackTreeNode_1.default.sentinel;
        }
        max() {
            const currentNode = this.root;
            if (this.isEmpty()) {
                return null;
            }
            return currentNode.max();
        }
        min() {
            const currentNode = this.root;
            if (this.isEmpty()) {
                return null;
            }
            return currentNode.min();
        }
        reverseTreeWalk(callback, initialValue) {
            return this.reverseNodeWalk(this.root, callback, initialValue);
        }
        search(value) {
            let currentNode = this.root;
            while (currentNode !== this.sentinel && currentNode.value !== value) {
                if (this.comparator(currentNode.value, value)) {
                    currentNode = currentNode.left;
                }
                else {
                    currentNode = currentNode.right;
                }
            }
            return currentNode === this.sentinel ? null : currentNode;
        }
        successor(value) {
            let node = this.search(value);
            let ancestor;
            if (node === null || value === null) {
                return null;
            }
            if (node.right !== this.sentinel) {
                return node.right.min();
            }
            ancestor = node.parent;
            while (ancestor !== this.sentinel && ancestor.right === node) {
                node = ancestor;
                ancestor = node.parent;
            }
            return ancestor === this.sentinel ? null : ancestor;
        }
        addFixup(node) {
            while (node.parent.isRed()) {
                if (node.parent.isLeftChild()) {
                    let parentSibling = node.parent.parent.right;
                    if (parentSibling.isRed()) {
                        node.parent.setBlack();
                        parentSibling.setBlack();
                        node.parent.parent.setRed();
                        node = node.parent.parent;
                    }
                    else {
                        if (node.isRightChild()) {
                            node = node.parent;
                            this.leftRotate(node);
                        }
                        node.parent.setBlack();
                        node.parent.parent.setRed();
                        this.rightRotate(node.parent.parent);
                    }
                }
                else {
                    let parentSibling = node.parent.parent.left;
                    if (parentSibling.isRed()) {
                        node.parent.setBlack();
                        parentSibling.setBlack();
                        node.parent.parent.setRed();
                        node = node.parent.parent;
                    }
                    else {
                        if (node.isLeftChild()) {
                            node = node.parent;
                            this.rightRotate(node);
                        }
                        node.parent.setBlack();
                        node.parent.parent.setRed();
                        this.leftRotate(node.parent.parent);
                    }
                }
            }
            this.root.setBlack();
        }
        inorderNodeWalk(node, callback, previousValue) {
            if (node !== this.sentinel) {
                previousValue = this.inorderNodeWalk(node.left, callback, previousValue);
                previousValue = callback(previousValue, node.value);
                previousValue = this.inorderNodeWalk(node.right, callback, previousValue);
                return previousValue;
            }
            else {
                return previousValue;
            }
        }
        leftRotate(node) {
            const promotedNode = node.right;
            node.right = promotedNode.left;
            if (promotedNode.left !== this.sentinel) {
                promotedNode.left.parent = node;
            }
            promotedNode.parent = node.parent;
            if (node.parent === this.sentinel) {
                this.root = promotedNode;
            }
            else if (node.isLeftChild()) {
                node.parent.left = promotedNode;
            }
            else {
                node.parent.right = promotedNode;
            }
            promotedNode.left = node;
            node.parent = promotedNode;
        }
        reverseNodeWalk(node, callback, previousValue) {
            if (node !== this.sentinel) {
                previousValue = this.reverseNodeWalk(node.right, callback, previousValue);
                previousValue = callback(previousValue, node.value);
                previousValue = this.reverseNodeWalk(node.left, callback, previousValue);
                return previousValue;
            }
            else {
                return previousValue;
            }
        }
        rightRotate(node) {
            const promotedNode = node.left;
            node.left = promotedNode.right;
            if (promotedNode.right !== this.sentinel) {
                promotedNode.right.parent = node;
            }
            promotedNode.parent = node.parent;
            if (node.parent === this.sentinel) {
                this.root = promotedNode;
            }
            else if (node.isRightChild()) {
                node.parent.right = promotedNode;
            }
            else {
                node.parent.left = promotedNode;
            }
            promotedNode.right = node;
            node.parent = promotedNode;
        }
    }
    exports.default = RedBlackTree;
});
//# sourceMappingURL=RedBlackTree.js.map